import time

import xbmcgui

import common
from smarty_api.program_api import ProgramAPIClient


class ControlID:
        labelTitle = 341
        imgChannel = 342
        labelChannel = 343

        btPrev = 344
        labelDay = 345
        btNext = 346
        btExit = 347

        labelEPGTitle = 240
        labelEPGDescription = 241
        labelEPGRating = 242
        imgPreview = 243

        list = 181


def open_epg_window(channel):
    window = EPGWindow("smarty.tv.epg.xml", common.get_addon_path(),
                       common.AddonSettings.get_skin_name())
    window.init(channel, common.KODI_ADDON)
    program = window.programSelected
    del window
    return program


# noinspection PyPep8Naming
class EPGWindow(xbmcgui.WindowXMLDialog):
    DEFAULT_ARCHIVE_ICON = 'epg_record_icon.png'

    def __init__(self, *args, **kwargs):
        super(EPGWindow, self).__init__(self, *args, **kwargs)

        self.addon = None
        self.title = ""
        self.channel = None
        self.day = None
        self.client = None

        self.programSelected = None

    def init(self, channel, addon):
        client = ProgramAPIClient()
        self.addon = addon
        self.title = addon.getLocalizedString(32300)
        self.channel = channel
        self.day = common.get_today_date()
        self.client = client
        self.doModal()

    def onInit(self):
        common.log_info("Window open", "infoWindow.onInit")

        super(EPGWindow, self).onInit()

        title = self.getControl(ControlID.labelTitle)
        title.setLabel(self.title)

        channelTitle = self.getControl(ControlID.labelChannel)
        channelTitle.setLabel(self.channel.name)

        imgChannel = self.getControl(ControlID.imgChannel)
        imgChannel.setImage(self.channel.iconURL)

        self.fillEPGList(True)

    def onArchiveSelected(self):
        program = self.getSelectedProgram()
        if program is None or not program.has_record:
            if program is None:
                common.log_warning("No program found", "infoWindow.onArchiveSelected")
            return

        common.log_info("Program selected, close window", "infoWindow.onArchiveSelected")
        self.programSelected = program
        self.close()

    def onAction(self, action):
        controlID = self.getFocusId()

        if action == xbmcgui.ACTION_MOVE_LEFT:
            self.prevDay()
        elif action == xbmcgui.ACTION_MOVE_RIGHT:
            self.nextDay()
        elif action == xbmcgui.ACTION_MOVE_DOWN:
            if controlID == ControlID.list:
                self.fillEPG()
        elif action == xbmcgui.ACTION_MOVE_UP:
            if controlID == ControlID.list:
                self.fillEPG()
        elif action == xbmcgui.ACTION_SELECT_ITEM or action == xbmcgui.ACTION_MOUSE_LEFT_CLICK \
                or action == xbmcgui.ACTION_MOUSE_DOUBLE_CLICK:
            if controlID == ControlID.btExit:
                common.log_info("Window close", "infoWindow.onAction")
                self.close()
            elif controlID == ControlID.btPrev:
                self.prevDay()
            elif controlID == ControlID.btNext:
                self.nextDay()
            elif controlID == ControlID.list:
                if action == xbmcgui.ACTION_MOUSE_LEFT_CLICK:
                    self.fillEPG()
                else:
                    self.onArchiveSelected()

        xbmcgui.WindowXMLDialog.onAction(self, action)

    def setControlLabel(self, string, controlID):
        try:
            control = self.getControl(controlID)
            control.setLabel(string)
            common.log_debug("setting label, control %d" % controlID, "epgWindow.setControlLabel")
        except:
            common.log_warning("error setting label, control %d" % controlID, "epgWindow.setControlLabel")

    def getEPG(self):
        return self.client.get_programs(self.channel.id, self.day)

    def getSelectedProgram(self):
        try:
            item = self.getControl(ControlID.list).getSelectedItem()
            program_id = int(item.getProperty('program_id'))
        except:
            return None
        return self.client.get_program_by_id(self.channel.id, self.day, program_id)

    def clearEPGList(self):
        control = self.getControl(ControlID.list)
        control.reset()

    def clearEPG(self):
        self.setControlLabel('', ControlID.labelEPGTitle)
        self.setControlLabel('', ControlID.labelEPGDescription)
        self.setControlLabel('', ControlID.labelEPGRating)
        imgPreview = self.getControl(ControlID.imgPreview)
        imgPreview.setImage('')

    def _makeEPGListItem(self, program):
        li = xbmcgui.ListItem(label=program.name, label2=program.description)
        timeStr = common.convert_timestamp_to_str(program.beginTime)
        li.setProperty('program_id', str(program.id))
        li.setProperty('time', timeStr)
        if program.has_record:
            li.setProperty('archive_icon', EPGWindow.DEFAULT_ARCHIVE_ICON)
        else:
            li.setProperty('archive_icon', '')
        return li

    def fillEPGList(self, selectNowProgram):
        programs = self.getEPG()        
            
        strDay = common.convert_date_to_str(self.day)
        self.setControlLabel(strDay, ControlID.labelDay)        
        
        self.clearEPGList()        

        nowItem = 0
        currentTime = time.time()
        epgList = self.getControl(ControlID.list)

        self.clearEPG()

        if programs is not None and len(programs) > 0:
            for program in programs:
                if program.beginTime <= currentTime <= program.endTime:
                    nowItem = epgList.size()
                epgList.addItem(self._makeEPGListItem(program))

            if selectNowProgram:
                epgList.selectItem(nowItem)
            else:
                epgList.selectItem(0)
            self.fillEPG()
        else:
            self.setControlLabel(self.addon.getLocalizedString(32301), ControlID.labelEPGTitle)
        self.setFocusId(ControlID.list)

    def fillEPG(self):
        program = self.getSelectedProgram()
        if program is None:
            return

        rating = "%d+" % program.rating if program.rating > 0 else ''

        self.setControlLabel(program.name, ControlID.labelEPGTitle)
        self.setControlLabel(program.description, ControlID.labelEPGDescription)
        self.setControlLabel(rating, ControlID.labelEPGRating)
        imgPreview = self.getControl(ControlID.imgPreview)
        imgPreview.setImage(program.previewURL)
        self.descriptionUp(len(program.previewURL) == 0)

    def descriptionUp(self, isUp):
        descriptionLabel = self.getControl(ControlID.labelEPGDescription)
        if isUp:
            descriptionLabel.setPosition(0, 65)
            descriptionLabel.setHeight(415)
        else:
            descriptionLabel.setPosition(0, 205)
            descriptionLabel.setHeight(275)

    def prevDay(self):
        self.day = common.add_days(self.day, -1)
        self.fillEPGList(False)

    def nextDay(self):
        self.day = common.add_days(self.day, 1)
        self.fillEPGList(False)
